# 
# Clear all conditional flags
unset sgi_mode
unset pgs_daac
unset pgs_f90_comp
unset pgs_nag_flag
# set the toolkit home directory environment variable
# SDP installation done on Thu Dec  6 10:13:43 MST 2001 
# 

setenv PGSHOME /usr/local/lib/SDPTK-Linux-5.2.5
set sgi_mode=32			# set SGI for standard mode
 
#-----------------------------------------------------------------------------
# file:	
# 	pgs-dev-env.csh
#
# description:
# 	This file defines environment variables used by SDP (PGS) Toolkit.
# 	This version is for use under the C shell (csh).
#
# usage:
# 	This file should be called from your .cshrc file with the line:
#
# 	    source <toolkit-home-dir>/bin/pgs-dev-env.csh
#	
# 	where <toolkit-home-dir> is the full path of the toolkit home directory.
#
# author:
# 	Mike Sucher / A.R.C.
#       Guru Tej S. Khalsa / Applied Research Corporation
#
# notes:
# 	1) This file is compatible with the following platforms:
# 	   Sun, SGI, HP-9000, IBM RS-6000, DEC Alpha and Cray.
# 	   It automatically figures out which platform you are on,
# 	   and sets environment variables accordingly.
# 	2) (OPTIONAL) if HDF is installed on your system, you may 
# 	   wish to edit the line near the end of this file that sets
# 	   up the environment variable HDFHOME.
# 	3) This file adds the PGS bin (and optionally the HDF bin) 
#	   directory to the 'path'  environment variable.
# 	4) This file defines a variable called pgs_path which contains
# 	   all the directories likely to be needed on a given machine
# 	   type, including the PGS (and HDF) bin directories.  Users 
# 	   may choose to set their path variable with:
# 	   
# 	           set path=(<user-path-additions> $pgs_path )
#
# 	   where <user-path-additions> is an optional list of other
# 	   directories added to the search path.
#
# history:
#	20-Apr-1994 MES  Initial version
#	31-Aug-1994 MES  Update for Release 3
# 	13-Sep-1994 MES  Added variables for Process Control tools
# 	14-Sep-1994 MES  Updated path for sun4 to include 
# 			   /usr/local/lang and /usr/lang
# 	26-Sep-1994 MES  Updated variable PGS_PC_INFO_FILE to point to the file
# 			 $PGSRUN/PCF.v3
# 	09-Feb-1995 MES  Updated variable PGS_PC_INFO_FILE to point to the file
# 			 $PGSRUN/PCF.v4
# 	03-May-1995 MES  Updated variable PGS_PC_INFO_FILE to point to the file
# 			 $PGSRUN/PCF.v5
# 			 Removed referenced to obsolete environment variables
# 			 no longer referenced by the Process Control tools.
# 	09-May-1995 MES  Updated compiler flags for Release 5
# 			   hp:                        ibm:
# 			   C_CFH:   remove -Dextname  C_CFH:   remove -Dextname
# 			   F77_CFH: remove +ppu       F77_CFH: remove -qextname
# 			 Note: this change was made to facilitate binding with
# 			 heritage code libraries.  Caution:  the cfortran.h 
# 			 bindings can ONLY be used with mixed-case C function
# 			 names.  Otherwise the bindings will not work.
#	15-Jun-1995 GTSK Added support for IRIX64, the sgi 64 bit O/S.
#			 This script will install the toolkit in 32 bit mode.
#	10-Jul-1995 MES  Removed flags "-h stdc" from Cray CFLAGS.
#	18-Aug-1995 MES  Revised environment for formal track directory
# 			 structure used by the DAAC version Toolkit for IR-1.
# 	31-Aug-1995 MES  Updated IRIX64 OS support to recognize both 32 and 64
# 			 bit modes.  Default is is 32-bit.  New BRAND setting
# 			 sgi64 supports the SGI 64 bit mode.
# 	30-Oct-1995 MES  Updates for IR-1 release:
# 			 - add support for SGI compiled -n32 ($BRAND = sgi32)
# 			 - formal directory structure is now set by pgs_formal
# 			 - revise PGSRUN: no longer architecture-specific
#  			 - set PGS_PC_INFO_FILE to $PGSRUN/$BRAND/PCF.ir1
# 	21-Nov-1995 MES  Handle the DAAC toolkit compilation.
# 	22-Nov-1995 MES  Add code to handle FORTRAN-90 compiler.
# 	15-Apr-1996 MES  Updates for Release A
#  			 - clear DAAC version env variables, if not used
#  			 - set PGS_PC_INFO_FILE to $PGSRUN/$BRAND/PCF.relA
# 	26-Aug-1996 MES  Updates for Release A
#  			 - update DAAC version logic
#  			 - update HDF environment variables setup
# 
#-----------------------------------------------------------------------------

if (! $?sgi_mode ) then		# by default, SGI mode is standard 32-bit
    set sgi_mode=32
endif
if (! $?pgs_daac ) then 	# by default, DAAC toolkit version disabled
    set pgs_daac=0
endif
if (! $?pgs_f90_comp ) then 	# by default, no FORTRAN-90 compiler
    set pgs_f90_comp=""
endif
if (! $?pgs_nag_flag ) then 	# by default, not using NAG FORTRAN-90
    set pgs_nag_flag=0
endif
if (! $?use_flavor ) then 	# by default, do not use "flavors"
    set use_flavor=0
endif

set user_path = ( $path )	# save user path

# set path to a base subset of directories, allowing startup on unknown host
# note: once the host has been determined the path is appropriately customized

set path=(/usr/local/bin /bin /usr/bin /etc /usr/etc /usr/ucb /usr/bin/X11)


# get operating system type, login name
# special cases: SCO and Cray  - uname works differently,

setenv MACHINE `uname -m | awk '{print $1}'`	# needed on Cray & SCO

switch ( "$MACHINE" )

    case "i386":        			# SCO box
	setenv OSTYPE sco386
        breaksw

    case "CRAY":    				# CRAY
	setenv OSTYPE UNICOS
        breaksw

    default:					# everybody else
	setenv OSTYPE `uname`
        breaksw

endsw

set user=`id | cut -d\( -f2 | cut -d\) -f1`
if ($?LOGNAME == 0) setenv LOGNAME $user	# make sure $LOGNAME is defined
setenv USER $LOGNAME				# make sure $USER is defined


# set machine-dependent environment variables:
# 	HOST  the host name of this machine
# 	BRAND used by other achitecture-specific code
# 	path  the execution search path exported to PATH

switch ( "$OSTYPE" )

    case "AIX": 
	set path=(/usr/local/bin /bin /usr/bin /etc /usr/etc /usr/ucb /usr/bin/X11 /usr/ccs/bin /usr/sbin)
	setenv HOST `hostname`
	setenv BRAND ibm
        breaksw

    case "HP-UX": 
	set path=(/usr/local/bin /bin /usr/bin /etc /usr/etc /usr/bin/X11)
	setenv HOST `hostname`
	setenv BRAND hp 
        breaksw

    case "IRIX":  
	set path=(/usr/local/bin /bin /usr/bin /etc /usr/etc /usr/bsd /usr/bin/X11 /usr/sbin)
	setenv HOST `hostname`
	setenv BRAND sgi 
        breaksw

    case "IRIX64":  
	set path=(/usr/local/bin /bin /usr/bin /etc /usr/etc /usr/bsd /usr/bin/X11 /usr/sbin)
	setenv HOST `hostname`
	if ("$sgi_mode" == 32) then
	    setenv BRAND sgi
        else if ("$sgi_mode" == 64) then
	    setenv BRAND sgi64
        else if ("$sgi_mode" == n32) then
	    setenv BRAND sgi32
        else
	    setenv BRAND sgi
        endif
        breaksw

    case "Linux": 
	set path=(/usr/local/bin /bin /usr/bin /etc /usr/etc /usr/X11/bin)
	setenv HOST `hostname`
	setenv BRAND linux 
        breaksw

    case "OSF1":  
	set path=(/usr/local/bin /bin /usr/bin /etc /usr/etc /usr/ucb /usr/bin/X11 /usr/ccs/bin /usr/sbin)
	setenv HOST `hostname -s`
	setenv BRAND dec 
        breaksw

    case "sco386": 
	set path=(/usr/local/bin /bin /usr/bin /etc /usr/etc /usr/bin/X11)
	setenv HOST `hostname -s`
	setenv BRAND sco 
        breaksw

    case "SunOS": 
	# distinguish between SunOS 5.x and 4.x versions
	if (`uname -r | awk -F. '{print $1}'` == "5") then 
	    setenv BRAND sun5			# release V5.x SunOS
	    set path=(/usr/local/bin /opt/SUNWspro/bin /bin /usr/bin /etc /usr/etc /usr/ucb /usr/openwin/bin /usr/openwin/demo /usr/ccs/bin /usr/sbin)
	else                                
	    setenv BRAND sun4			# release V4.x SunOS
	    set path=(/usr/local/bin /usr/local/lang /usr/lang /bin /usr/bin /etc /usr/etc /usr/ucb /usr/openwin/bin /usr/openwin/demo)
	endif
	setenv HOST `hostname`
        breaksw

    case "UNICOS": 
	set path=(/usr/local/bin /bin /usr/bin /etc /usr/ucb /usr/bin/X11)
	setenv HOST `hostname`
	setenv BRAND cray 
        breaksw

    default:
	echo "Operating system: $OSTYPE not supported"
	echo "This release of the PGS Toolkit supports: "
	echo "   Sun, SGI HP-9000 IBM-6000 DEC-Alpha and Cray/Unicos "
        breaksw

endsw




# set machine-dependent compilers and compilation switches:
#
#

switch ($BRAND)

    case cray:
	setenv CC cc 			# C compiler
	setenv CFLAGS "-O" 		# default C flags (optimize, ansi)
	setenv C_CFH "-DCRAYFortran"    # C/cfortran.h called from FORTRAN
	setenv CFHFLAGS "$CFLAGS $C_CFH" # CFLAGS + C_CFH
	setenv F77 cf77			# FORTRAN compiler
	setenv F77FLAGS "-O" 		# common FORTRAN flags
        setenv F77_CFH ""               # FORTRAN callable from C w/ cfortran.h
	setenv F77_C_CFH ""          	# calling C w/ cfortran.h
	setenv F77_C_LIB ""		# C lib called by FORTRAN main 
	setenv HDFSYS UNICOS		# system type as defined by HDF
	breaksw

    case dec:
	setenv CC cc 			# C compiler
	setenv CFLAGS "-O -std"		# default C flags (optimize, ansi)
	setenv C_CFH "-DDECFortran"	# C w/ cfortran.h callable from FORTRAN
	setenv CFHFLAGS "$CFLAGS $C_CFH" # CFLAGS + C_CFH
	setenv F77 f77 			# FORTRAN compiler
	setenv F77FLAGS "-O" 		# common FORTRAN flags
        setenv F77_CFH ""               # FORTRAN callable from C w/ cfortran.h
	setenv F77_C_CFH ""             # calling C w/ cfortran.h
	setenv F77_C_LIB ""		# C lib called by FORTRAN main
	setenv HDFSYS DEC_ALPHA		# system type as defined by HDF
	breaksw

    case hp:
	setenv CC cc 			# C compiler
	setenv CFLAGS "-O -Ae" 		# default C flags (optimize, ansi)
	setenv C_CFH "" 		# C w/ cfortran.h callable from FORTRAN
	setenv CFHFLAGS "$CFLAGS $C_CFH" # CFLAGS + C_CFH
	setenv F77 fort77		# FORTRAN compiler
	setenv F77FLAGS "-O" 		# common FORTRAN flags
        setenv F77_CFH ""               # FORTRAN callable from C w/ cfortran.h
	setenv F77_C_CFH ""          	# calling C w/ cfortran.h
	setenv F77_C_LIB ""		# C lib called by FORTRAN main
	setenv HDFSYS HP9000		# system type as defined by HDF
	breaksw

    case ibm:
	setenv CC cc 			# C compiler
	setenv CFLAGS "-O -qlanglvl=ansi" # default C flags (optimize, ansi)
	setenv C_CFH "" 		# C w/ cfortran.h callable from FORTRAN
	setenv CFHFLAGS "$CFLAGS $C_CFH" # CFLAGS + C_CFH
	setenv F77 xlf 			# FORTRAN compiler
	setenv F77FLAGS "-O" 		# common FORTRAN flags
        setenv F77_CFH ""               # FORTRAN callable from C w/ cfortran.h
	setenv F77_C_CFH ""          	# calling C w/ cfortran.h
	setenv F77_C_LIB ""		# C lib called by FORTRAN main
	setenv HDFSYS IBM6000		# system type as defined by HDF
	breaksw

    case linux:
	setenv CC gcc 			# C compiler
	setenv CFLAGS "-O -ansi"	# default C flags (optimize, ansi)
	setenv C_CFH "-Df2cFortran"	# C w/ cfortran.h callable from FORTRAN
	setenv CFHFLAGS "$CFLAGS $C_CFH" # CFLAGS + C_CFH
	setenv F77 f77			# FORTRAN compiler
	setenv F77FLAGS "-O" 		# common FORTRAN flags
        setenv F77_CFH ""               # FORTRAN callable from C w/ cfortran.h
	setenv F77_C_CFH ""          	# calling C w/ cfortran.h
	setenv F77_C_LIB ""		# C lib called by FORTRAN main
	setenv HDFSYS LINUX		# system type as defined by HDF
	breaksw

    case sco:
	setenv CC cc 			# C compiler
	setenv CFLAGS "-O -posix"	# default C flags (optimize, ansi)
	setenv C_CFH "-Df2cFortran"	# C w/ cfortran.h callable from FORTRAN
	setenv CFHFLAGS "$CFLAGS $C_CFH" # CFLAGS + C_CFH
	setenv F77 ""			# FORTRAN compiler
	setenv F77FLAGS "-O" 		# common FORTRAN flags
        setenv F77_CFH ""               # FORTRAN callable from C w/ cfortran.h
	setenv F77_C_CFH ""          	# calling C w/ cfortran.h
	setenv F77_C_LIB ""		# C lib called by FORTRAN main
	setenv HDFSYS SCO		# system type as defined by HDF
	breaksw

    case sgi:
	if ($OSTYPE == "IRIX64") then
		setenv CC "cc -32"	# C compiler (32 bit)
		setenv F77 "f77 -32"	# FORTRAN compiler (32 bit)
	else
                setenv CC cc		# C compiler
                setenv F77 f77		# FORTRAN compiler
	endif
	setenv CFLAGS "-O -xansi -D_POSIX_SOURCE"	# default C flags (optimize, ansi)
	setenv C_CFH ""	 		# C w/ cfortran.h callable from FORTRAN
	setenv CFHFLAGS "$CFLAGS $C_CFH" # CFLAGS + C_CFH
	setenv F77FLAGS "-O" 		# common FORTRAN flags
        setenv F77_CFH ""               # FORTRAN callable from C w/ cfortran.h
	setenv F77_C_CFH ""          	# calling C w/ cfortran.h
	setenv F77_C_LIB ""		# C lib called by FORTRAN main
	setenv HDFSYS IRIX		# system type as defined by HDF
	breaksw

    case sgi32:
	setenv CC "cc -n32"		# C compiler (new-style 32 bit)
	setenv F77 "f77 -n32"		# FORTRAN compiler (new-style 32 bit)
	setenv CFLAGS "-O -xansi -D_POSIX_SOURCE"	# default C flags (optimize, ansi)
	setenv C_CFH ""	 		# C w/ cfortran.h callable from FORTRAN
	setenv CFHFLAGS "$CFLAGS $C_CFH" # CFLAGS + C_CFH
	setenv F77FLAGS "-O" 		# common FORTRAN flags
        setenv F77_CFH ""               # FORTRAN callable from C w/ cfortran.h
	setenv F77_C_CFH ""          	# calling C w/ cfortran.h
	setenv F77_C_LIB ""		# C lib called by FORTRAN main
	setenv HDFSYS IRIX		# system type as defined by HDF
	breaksw

    case sgi64:
	set cpu_type=`hinv | fgrep CPU | head -1 | cut -d' ' -f3 | cut -b2`
	if ("$cpu_type" == "4") then
                setenv CC "cc -64 -mips3"       # C compiler (R4?00 chip)
                setenv F77 "f77 -64 -mips3"     # FORTRAN compiler (R4?00 chip)
        else
                setenv CC "cc -64"      # C compiler
                setenv F77 "f77 -64"    # FORTRAN compiler
        endif
	setenv CFLAGS "-O -xansi -D_POSIX_SOURCE"	# default C flags (optimize, ansi)
	setenv C_CFH ""	 		# C w/ cfortran.h callable from FORTRAN
	setenv CFHFLAGS "$CFLAGS $C_CFH" # CFLAGS + C_CFH
	setenv F77FLAGS "-O" 		# common FORTRAN flags
        setenv F77_CFH ""               # FORTRAN callable from C w/ cfortran.h
	setenv F77_C_CFH ""          	# calling C w/ cfortran.h
	setenv F77_C_LIB ""		# C lib called by FORTRAN main
	setenv HDFSYS IRIX		# system type as defined by HDF
	breaksw

    case sun4:
	setenv CC acc			# C compiler
	setenv CFLAGS "-O -Xa" 		# default C flags (optimize, ansi)
	setenv C_CFH "-DsunFortran"	# C w/ cfortran.h callable from FORTRAN
	setenv CFHFLAGS "$CFLAGS $C_CFH" # CFLAGS + C_CFH
	setenv F77 f77 			# FORTRAN compiler
	setenv F77FLAGS "-O" 		# common FORTRAN flags
        setenv F77_CFH ""               # FORTRAN callable from C w/ cfortran.h
	setenv F77_C_CFH ""           	# calling C w/ cfortran.h
	setenv F77_C_LIB "-lm" 		# C lib called by FORTRAN main
	setenv HDFSYS SUN		# system type as defined by HDF
	breaksw

    case sun5:
	setenv CC cc			# C compiler
	setenv CFLAGS "-O -Xa" 		# default C flags (optimize, ansi)
	setenv C_CFH "-DsunFortran"	# C w/ cfortran.h callable from FORTRAN
	setenv CFHFLAGS "$CFLAGS $C_CFH" # CFLAGS + C_CFH
	setenv F77 f77 			# FORTRAN compiler
	setenv F77FLAGS "-O" 		# common FORTRAN flags
        setenv F77_CFH ""               # FORTRAN callable from C w/ cfortran.h
	setenv F77_C_CFH ""           	# calling C w/ cfortran.h
	setenv F77_C_LIB "-lm" 		# C lib called by FORTRAN main
	setenv HDFSYS SUN		# system type as defined by HDF
	breaksw

    default:
	setenv CC cc			# C compiler
	setenv CFLAGS "-O" 		# default C flags (optimize)
	setenv C_CFH ""	        	# C w/ cfortran.h callable from FORTRAN
	setenv CFHFLAGS "$CFLAGS $C_CFH" # CFLAGS + C_CFH
	setenv F77 f77 			# FORTRAN compiler
	setenv F77FLAGS "-O" 		# common FORTRAN flags
        setenv F77_CFH ""               # FORTRAN callable from C w/ cfortran.h
	setenv F77_C_CFH ""           	# calling C w/ cfortran.h
	setenv F77_C_LIB "-lm" 		# C lib called by FORTRAN main
	setenv HDFSYS unknown		# system type as defined by HDF
	breaksw
endsw


# 
# set up environment to handle FORTRAN-90 compiler
#

if ("$pgs_f90_comp" != "") then		# using FORTRAN-90

    setenv F77 "$pgs_f90_comp"

    if ("$pgs_nag_flag" == "1") then	# using NAG f90
        setenv C_CFH "$C_CFH -DNAGf90F"
        setenv CFHFLAGS "$CFLAGS $C_CFH"
    endif

endif


# copy the machine-specific path to variable pgs_path

set pgs_path = ($path)

# set PGS-related environment variables
# these may be referred to in makefiles and on compiler command lines

if ( $?PGSHOME ) then

# set up base set of PGS Toolkit directory variables.

    setenv PGSBIN 	$PGSHOME/bin/$BRAND		# exectuable files
    setenv PGSDAT 	$PGSHOME/database/$BRAND	# database files
    setenv PGSINC 	$PGSHOME/include		# include (header) files
    setenv PGSLIB 	$PGSHOME/lib/$BRAND		# library files
    setenv PGSMSG 	$PGSHOME/message		# SMF message files
    setenv PGSOBJ 	$PGSHOME/obj/$BRAND		# object files
    setenv PGSRUN 	$PGSHOME/runtime		# runtime work files
    setenv PGSSRC 	$PGSHOME/src			# toolkit source files
    setenv PGSTST 	$PGSHOME/test			# test source files


    if ("$use_flavor" == 1) then

        setenv PGSBIN 	${PGSBIN}_${flavor}		# exectuable files
        setenv PGSLIB 	${PGSLIB}_${flavor} 		# library files
        setenv PGSOBJ 	${PGSOBJ}_${flavor}		# object files

    endif

# update path variables

    set path  = ($path $PGSBIN)			# add PGSBIN to pgs  path
    set pgs_path  = ($pgs_path $PGSBIN)		# add PGSBIN to pgs  path
    set user_path = ($user_path $PGSBIN)	# add PGSBIN to user path

# set up variables needed by Process Control (PC) tools. 
# user may customize these in their private .cshrc file

    setenv PGS_PC_INFO_FILE    	$PGSRUN/$BRAND/PCF.relB0 
    if ( $use_flavor == 1 ) then
	if ( $flavor =~ *daac* ) then
	    setenv PGS_PC_INFO_FILE    	$PGSRUN/$BRAND/PCF.relB0_daac
	else
	    setenv PGS_PC_INFO_FILE    	$PGSRUN/$BRAND/PCF.relB0_scf
	endif
    endif



else

    echo "You must first set the environment variable PGSHOME"

endif


# set HDF-related environment variables
# these may be referred to in makefiles and on compiler command lines
# use the command 'sethdf <hdf-home-directory> to override the default

####
#
#
# if your site has HDF installed:
#    uncomment the next line and edit it to properly 
#    set your HDF home directory
#
####

setenv HDFHOME ${PGSHOME}/hdf/linux/HDF4.1r1	# set HDF home directory

if ( $use_flavor == 1 ) then
    if ( $flavor =~ *debug ) then
	set hdfhome=`echo $HDFHOME | sed "s/${BRAND}/${BRAND}_debug/"`
        if ( -d $hdfhome ) then
	    setenv HDFHOME $hdfhome
        endif
	unset hdfhome
    endif
endif

#
# Set HDF directories:
#
# - first look in $HDFHOME
# - if not found, default to $HDFHOME/hdf
#

if ($?HDFHOME) then

    if ( -d $HDFHOME/bin ) then		# HDF utilities
        setenv HDFBIN $HDFHOME/bin
    else
        setenv HDFBIN $HDFHOME/hdf/bin
    endif

    if ( -d $HDFHOME/include ) then	# HDF header files
        setenv HDFINC $HDFHOME/include
    else
        setenv HDFINC $HDFHOME/hdf/include
    endif

    if ( -d $HDFHOME/lib ) then		# HDF libraries
        setenv HDFLIB $HDFHOME/lib
    else
        setenv HDFLIB $HDFHOME/hdf/lib
    endif



    set pgs_path  = ($pgs_path  $HDFBIN)	# add HDFBIN to pgs  path
    set user_path = ($user_path $HDFBIN)	# add HDFBIN to user path

endif


####
#
#
# if your site has HDF-EOS installed:
#    uncomment the next line and edit it to properly 
#    set your HDF-EOS home directory
#
####

setenv HDFEOS_HOME ${PGSHOME}/hdfeos	# set HDF-EOS home directory

#
# Set HDF-EOS directories:
#

if ($?HDFEOS_HOME) then

    setenv HDFEOS_INC $HDFEOS_HOME/include
    setenv HDFEOS_LIB $HDFEOS_HOME/lib/$BRAND
    setenv HDFEOS_BIN $HDFEOS_HOME/bin/$BRAND

    if ( $use_flavor == 1 ) then
	if ( $flavor =~ *debug ) then
	    if ( -d ${HDFEOS_LIB}_debug ) then
		setenv HDFEOS_LIB ${HDFEOS_LIB}_debug
		setenv HDFEOS_BIN ${HDFEOS_BIN}_debug
	    endif
	endif
    endif

endif

#
# check to see if DAAC version of the toolkit was installed
#

if ( $?ADD_IFLAGS ) setenv ADD_IFLAGS "" # make sure old values are gone
if ( $?ADD_LFLAGS ) setenv ADD_LFLAGS "" # make sure old values are gone
if ( $?ADD_LIBS )   setenv ADD_LIBS ""   # make sure old values are gone

if ("$pgs_daac" == "1") then

    setenv CFLAGS "$CFLAGS -DPGS_DAAC"
    setenv CFHFLAGS "$CFHFLAGS -DPGS_DAAC"

endif

if ( $?ECS_VERSION ) then

    setenv VFLAG "'-DPGSd_ECS_VERSION="'"${ECS_VERSION}"'"'"

endif

#
# restore augmented user path
#
set path = ( $user_path )


# done
