/*-------------------------------------------------------------------------*/
/*                                                                         */
/*  COPYRIGHT[copyright mark] 1996, Hughes Aircraft Company, its vendors,  */
/*  and suppliers.  ALL RIGHTS RESERVED.                                   */
/*                                                                         */
/*-------------------------------------------------------------------------*/
/***************************************************************************
BEGIN_FILE_PROLOG:

FILENAME:  
  	 PGS_MET_Remove.c
 
DESCRIPTION:
	This file contains PGS_MET_Remove() which frees the memory held by
	the MCF 

AUTHOR:
  	Alward N.Siyyid / EOSL
        Carol S. W. Tsai / Space Applications Corporation

HISTORY:
  	01-JUN-1995 	ANS 	Initial version
	13-July-95      ANS     Improved Fortran example
	25-July-95	ANS	Fixed ECSed01028
	20-MAR-96	ANS 	update for tk5+
        01-Oct-97       CSWT    Deleted code that removes the aggregate node in 
                                the memory (This is for NCR ECSed09220 about 
                                PGS_MET_GetPCAttr failure after PGS_MET_Remove 
                                and PGS_MET_GetPCAttr fails if pgs_met_init is 
                                called twice) 

END_FILE_PROLOG:
***************************************************************************/
/*----- includes ------*/

#include <stdio.h>
#include <CUC/odldef.h>
#include <CUC/odlinter.h>

#include <errno.h>
#include <PGS_MET.h>

/***************************************************************************
BEGIN_PROLOG:
 
	Removes MCF from the memory 
TITLE:
	Removes ODL representation of MCF file 
 
NAME:
        PGS_MET_Remove()
 
SYNOPSIS:        
C:
        #include "PGS_MET.h"

        PGSt_SMF_status
        PGS_MET_Remove()

FORTRAN:
         include "PGS_MET_13.f"
         include "PGS_MET.f"
         include "PGS_SMF.h"
         integer function pgs_met_remove()
 
DESCRIPTION:
	This routine removes ODL representation of MCF file plus some internal buffers used by the system
	
 
INPUTS:
	None

OUTPUTS:
        None
 
RETURNS:
	None
 
EXAMPLES:
C:

	This is an extract from the main example in PGS_MET_Init()

	PGS_MET_Remove();
        printf("SUCCESS\n");
        return 0;
FORTRAN:
 
	This is an extract from the main example in PGS_MET_Init()
	
	print *, ival, dval, datetime
        result = pgs_met_remove()
        print *, "SUCCESS"
 
        end
NOTES:
        This routine must be called by the user before exiting from his/her program
	It removes all the met handles initiated by the user
 
REQUIREMENTS:
        PGSTK-0430
 
DETAILS:
        NONE
 
GLOBALS:
	PGSg_MET_MasterNode 
FILES:
        NONE
 
FUNCTIONS_CALLED:
	RemoveAggregate
 
END_PROLOG:
***************************************************************************/
void
PGS_MET_Remove() /* removes MCF and the data dictionary from memory */
{
	extern AGGREGATE PGSg_MET_MasterNode[];
	extern AGGREGATE PGSg_MET_AggList[];
	extern PGSt_integer     PGSg_MET_NumOfMCF;

	/* removing MCF */

	PGSt_integer i;

	for(i = 0; i<= PGSg_MET_NumOfMCF; i++)
        {
        	PGSg_MET_MasterNode[i] = RemoveAggregate(PGSg_MET_MasterNode[i]);
	}

	PGSg_MET_NumOfMCF = -1;
}
