#-------------------------------------------------------------------------#
#                                                                         #
#  COPYRIGHT[copyright mark] 1996, Hughes Aircraft Company, its vendors,  #
#  and suppliers.  ALL RIGHTS RESERVED.                                   #
#                                                                         #
#-------------------------------------------------------------------------#
#-----------------------------------------------------------------------------
# file:         makefile for MET tools
#
# environment:  MPPDE, machine-independent, PGS directory structure
#
# environment variables dependencies:
#   compiler:  CC CFHFLAGS F77 F77_CFH
#   includes:  PGSINC
#   other:     PGSOBJ HDFSYS HDFINC
#
# author: Mike Sucher / Graham Bland / Megan E.D. Spencer
# history:
#       01-Apr-1994 MES Initial version
#	02-JAN-1995 ANS Adopted for GCT
#	04-Apr-1995 ANS Adopted for MET
# notes:
#       1) This file is intended for use in the Multi-Platform PGS Development
#          Environment (MPPDE) .  It depends on the PGS-defined toolkit
#          directory structure, and on environmental variables defined
#          by MPPDE startup files.  This is how machine independence
#          is maintained.
#       2) Target object files are moved to $(PGSOBJ)/MET
#
#----------------------------------------------------------------------------

#
# Set the name of this tool group
#

GRP= MET

# define sh shell (needed for embedded shell scripts)
SHELL=/bin/sh

# Name of the remove utility and flags
RM= /bin/rm
RMFLAGS= -f 
MFLAGS2= -f makefile.cpp
# define C preprocessor symbols
DFLAGS  = -D$(HDFSYS) -DPGS_MET_COMPILE

# path for #include directive
IFLAG   = -I$(PGSINC) -I$(PGSINC)/CUC -I$(HDFINC) 

# object files from this sub-group needed to build library

LIBOFILES = \
	$(PGSCPPO)/$(GRP)/PGS_MET_GetSetAttr.o \
	$(PGSCPPO)/$(GRP)/PGS_MET_GetSetAttrF.o \
	$(PGSCPPO)/$(GRP)/PGS_MET_Init.o \
	$(PGSCPPO)/$(GRP)/PGS_MET_SetAttr.o \
	$(PGSCPPO)/$(GRP)/PGS_MET_SetAttrF.o \
	$(PGSCPPO)/$(GRP)/PGS_MET_GetPCAttr.o \
	$(PGSCPPO)/$(GRP)/PGS_MET_GetPCAttrF.o \
	$(PGSCPPO)/$(GRP)/PGS_MET_GetConfigData.o \
	$(PGSCPPO)/$(GRP)/PGS_MET_GetConfigDataF.o \
	$(PGSCPPO)/$(GRP)/PGS_MET_Write.o \
	$(PGSCPPO)/$(GRP)/PGS_MET_Remove.o 
#
# targets
#

all:    $(LIBOFILES)

clean:
	$(RM) $(RMFLAGS) core *.o
	$(RM) $(RMFLAGS) $(LIBOFILES)

#
# compilation rules
#

$(PGSCPPO)/$(GRP)/PGS_MET_GetSetAttr.o: PGS_MET_GetSetAttr.c
	$(CPP) -c $(CPPFHFLAGS) $(DFLAGS) $(IFLAG) $? -o $@

$(PGSCPPO)/$(GRP)/PGS_MET_GetSetAttrF.o: PGS_MET_GetSetAttrF.c
	$(CPP) -c $(CPPFHFLAGS) $(DFLAGS) $(IFLAG) $? -o $@

$(PGSCPPO)/$(GRP)/PGS_MET_Init.o: PGS_MET_Init.c
	$(CPP) -c $(CPPFHFLAGS) $(DFLAGS) $(IFLAG) $? -o $@

$(PGSCPPO)/$(GRP)/PGS_MET_SetAttr.o: PGS_MET_SetAttr.c
	$(CPP) -c $(CPPFHFLAGS) $(DFLAGS) $(IFLAG) $? -o $@

$(PGSCPPO)/$(GRP)/PGS_MET_SetAttrF.o: PGS_MET_SetAttrF.c
	$(CPP) -c $(CPPFHFLAGS) $(DFLAGS) $(IFLAG) $? -o $@

$(PGSCPPO)/$(GRP)/PGS_MET_GetPCAttr.o: PGS_MET_GetPCAttr.c
	$(CPP) -c $(CPPFHFLAGS) $(DFLAGS) $(IFLAG) $? -o $@

$(PGSCPPO)/$(GRP)/PGS_MET_GetPCAttrF.o: PGS_MET_GetPCAttrF.c
	$(CPP) -c $(CPPFHFLAGS) $(DFLAGS) $(IFLAG) $? -o $@

$(PGSCPPO)/$(GRP)/PGS_MET_GetConfigData.o: PGS_MET_GetConfigData.c
	$(CPP) -c $(CPPFHFLAGS) $(DFLAGS) $(IFLAG) $? -o $@

$(PGSCPPO)/$(GRP)/PGS_MET_GetConfigDataF.o: PGS_MET_GetConfigDataF.c
	$(CPP) -c $(CPPFHFLAGS) $(DFLAGS) $(IFLAG) $? -o $@

$(PGSCPPO)/$(GRP)/PGS_MET_Write.o: PGS_MET_Write.c
	$(CPP) -c $(CPPFHFLAGS) $(DFLAGS) $(IFLAG) $? -o $@

$(PGSCPPO)/$(GRP)/PGS_MET_Remove.o: PGS_MET_Remove.c
	$(CPP) -c $(CPPFHFLAGS) $(DFLAGS) $(IFLAG) $? -o $@


# compile C to object
.c.o:
	$(MAKE) $(MFLAGS) $(PGSCPPO)/$(LIBOFILES)/$@
